/*
 * Copyright 2021 Layotto Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package grpc

import (
	"google.golang.org/grpc"
	mgrpc "mosn.io/mosn/pkg/filter/network/grpc"
)

type grpcOptions struct {
	api     API
	maker   NewServer
	options []grpc.ServerOption
}

type Option func(o *grpcOptions)

// add an api for grpc server.
// api CANNOT be nil.
func WithAPI(api API) Option {
	return func(o *grpcOptions) {
		o.api = api
	}
}

type NewServer func(api API, opts ...grpc.ServerOption) mgrpc.RegisteredServer

func WithNewServer(f NewServer) Option {
	return func(o *grpcOptions) {
		o.maker = f
	}
}

func WithGrpcOptions(options ...grpc.ServerOption) Option {
	return func(o *grpcOptions) {
		o.options = append(o.options, options...)
	}
}
